@@ -11,7 +11,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.34';
+$VERSION = '1.39';
 
 sub ProcessOcad($$$);
 sub ProcessJPEG_HDR($$$);
@@ -87,6 +87,11 @@ sub ProcessJPEG_HDR($$$);
         Name => 'InfiRayVersion',
         Condition => '$$valPt =~ /^....IJPEG\0/s',
         SubDirectory => { TagTable => 'Image::ExifTool::InfiRay::Version' },
+      }, {
+        Name => 'UniformResourceName',
+        Groups => { 1 => 'APP2' },
+        Condition => '$$valPt =~ /^urn:/',
+        Notes => 'used in Apple HDR images',
       }, {
         Name => 'PreviewImage',
         Condition => '$$valPt =~ /^(|QVGA\0|BGTH)\xff\xd8\xff\xdb/',
@@ -129,14 +134,26 @@ sub ProcessJPEG_HDR($$$);
         Name => 'FPXR', # (non-standard location written by some HP models)
         Condition => '$$valPt =~ /^FPXR\0/',
         SubDirectory => { TagTable => 'Image::ExifTool::FlashPix::Main' },
+      }, {
+        Name => 'QualcommDualCamera',
+        Condition => '$$valPt =~ /^Qualcomm Dual Camera Attributes/',
+        SubDirectory => { TagTable => 'Image::ExifTool::Qualcomm::DualCamera' },
       }, {
         Name => 'InfiRayFactory',
         Condition => '$$self{HasIJPEG}"',
         SubDirectory => { TagTable => 'Image::ExifTool::InfiRay::Factory' },
       }, {
-        Name => 'ThermalParams', # (written by DJI FLIR models)
+        Name => 'ThermalParams', # (written by some DJI FLIR models)
         Condition => '$$self{Make} eq "DJI" and $$valPt =~ /^\xaa\x55\x12\x06/',
         SubDirectory => { TagTable => 'Image::ExifTool::DJI::ThermalParams' },
+      }, {
+        Name => 'ThermalParams2', # (written by M3T)
+        Condition => '$$self{Make} eq "DJI" and $$valPt =~ /^(.{32})?.{32}\x2c\x01\x20\0/s',
+        SubDirectory => { TagTable => 'Image::ExifTool::DJI::ThermalParams2' },
+      }, {
+        Name => 'ThermalParams3', # (written by M30T)
+        Condition => '$$self{Make} eq "DJI" and $$valPt =~ /^.{32}\xaa\x55\x38\0/s',
+        SubDirectory => { TagTable => 'Image::ExifTool::DJI::ThermalParams3' },
       }, {
         Name => 'PreviewImage', # (eg. Samsung S1060)
         Notes => 'continued from APP3',
@@ -189,7 +206,7 @@ sub ProcessJPEG_HDR($$$);
         Groups => { 0 => 'APP6', 1 => 'DJI' },
         Notes => 'DJI Thermal Analysis Tool record',
         ValueConv => 'substr($val,7)',
-      # also seen Motorola APP6 "MMIMETA\0", with sub-types: AL3A,ALED,MMI0,MOTD,QC3A
+      # also seen Motorola APP6 "MMIMETA\0", with sub-types: AL3A,ALED,MMI0,MOTD,QC3A,LMB1
     }],
     APP7 => [{
         Name => 'Pentax',
@@ -207,6 +224,10 @@ sub ProcessJPEG_HDR($$$);
         Name => 'InfiRayOpMode',
         Condition => '$$self{HasIJPEG}',
         SubDirectory => { TagTable => 'Image::ExifTool::InfiRay::OpMode' },
+      }, {
+        Name => 'DJI-DBG',
+        Condition => '$$valPt =~ /^DJI-DBG\0/',
+        SubDirectory => { TagTable => 'Image::ExifTool::DJI::Info' },
     }],
     APP8 => [{
         Name => 'SPIFF',
@@ -216,6 +237,10 @@ sub ProcessJPEG_HDR($$$);
         Name => 'InfiRayIsothermal',
         Condition => '$$self{HasIJPEG}',
         SubDirectory => { TagTable => 'Image::ExifTool::InfiRay::Isothermal' },
+      }, {
+        Name => 'SEAL',
+        Condition => '$$valPt =~ /^SEAL\0/',
+        SubDirectory => { TagTable => 'Image::ExifTool::XMP::SEAL' },
     }],
     APP9 => [{
         Name => 'MediaJukebox',
@@ -225,21 +250,29 @@ sub ProcessJPEG_HDR($$$);
         Name => 'InfiRaySensor',
         Condition => '$$self{HasIJPEG}',
         SubDirectory => { TagTable => 'Image::ExifTool::InfiRay::Sensor' },
+      }, {
+        Name => 'SEAL',
+        Condition => '$$valPt =~ /^SEAL\0/',
+        SubDirectory => { TagTable => 'Image::ExifTool::XMP::SEAL' },
     }],
-    APP10 => {
+    APP10 => [{
         Name => 'Comment',
         Condition => '$$valPt =~ /^UNICODE\0/',
         Notes => 'PhotoStudio Unicode comment',
-    },
+      }, {
+        Name => 'HDRGainInfo', #PH (NC)
+        Condition => '$$valPt =~ /^AROT\0\0.{4}/s',
+        SubDirectory => { TagTable => 'Image::ExifTool::JPEG::HDRGainInfo' },
+    }],
     APP11 => [{
         Name => 'JPEG-HDR',
         Condition => '$$valPt =~ /^HDR_RI /',
         SubDirectory => { TagTable => 'Image::ExifTool::JPEG::HDR' },
-    },{
+      }, {
         Name => 'JUMBF',
         Condition => '$$valPt =~ /^JP/',
         SubDirectory => { TagTable => 'Image::ExifTool::Jpeg2000::Main' },
-        # Note: The recommended options for reading C2PA JUMBF metadata are "-G3 -b -j -u"
+        # Note: The suggested options for reading C2PA CAI JUMBF metadata are "-G3 -b -j -u"
     }],
     APP12 => [{
         Name => 'PictureInfo',
@@ -308,10 +341,23 @@ sub ProcessJPEG_HDR($$$);
             $$valPt =~ /~\0\x04\0zmie~\0\0\x0a.{8}[\x10\x18]\x08$/s
         },
         SubDirectory => { TagTable => 'Image::ExifTool::MIE::Main' },
+      }, {
+        Name => 'MPF',
+        SubDirectory => { TagTable => 'Image::ExifTool::MPF::Main' },
       }, {
         Name => 'Samsung',
         Condition => '$$valPt =~ /QDIOBS$/',
         SubDirectory => { TagTable => 'Image::ExifTool::Samsung::Trailer' },
+      }, {
+        Name => 'Vivo',
+        Condition => '$$valPt =~ /^(streamdata|vivo\{")/',
+        SubDirectory => { TagTable => 'Image::ExifTool::Trailer::Vivo' },
+      }, {
+        Name => 'OnePlus',
+        SubDirectory => { TagTable => 'Image::ExifTool::Trailer::OnePlus' },
+      }, {
+        Name => 'Google',
+        SubDirectory => { TagTable => 'Image::ExifTool::Trailer::Google' },
       }, {
         Name => 'EmbeddedVideo',
         Notes => 'extracted only when ExtractEmbedded option is used',
@@ -323,6 +369,9 @@ sub ProcessJPEG_HDR($$$);
         Name => 'NikonApp',
         Condition => '$$valPt =~ m(\0{6}/NIKON APP$)',
         Notes => 'contains editing information in XMP format',
+      }, {
+        Name => 'SonyHiddenData',
+        Condition => '$$valPt =~ /^\x55\x26\x11\x05\0/',
       }, {
         Name => 'PreviewImage',
         Condition => '$$valPt =~ /^\xff\xd8\xff/',
@@ -330,6 +379,21 @@ sub ProcessJPEG_HDR($$$);
     }],
 );
 
+# HDR gain information (ref PH)
+%Image::ExifTool::JPEG::HDRGainInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'APP10', 1 => 'AROT', 2 => 'Image' },
+    6 => {
+        Name => 'HDRGainCurveSize',
+        Format => 'int32u',
+    },
+    10 => {
+        Name => 'HDRGainCurve', # (NC)
+        Format => 'int32uRev[$val{6}]',
+        Binary => 1,
+    },
+);
+
 # JPS APP3 segment (ref http://paulbourke.net/stereographics/stereoimage/)
 %Image::ExifTool::JPEG::JPS = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
@@ -753,7 +817,7 @@ segments are included in the Image::ExifTool module itself.
 
 =head1 AUTHOR
 
-Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
